//Libaries Used-------------------------------------------------------------------------
#include <ax12.h>
#include <BioloidController.h>
#include <SPI.h>
#include <Pixy.h>
#include "poses.h"

Pixy pixy;

BioloidController bioloid = BioloidController(1000000);

//Program Variables---------------------------------------------------------------------
const int SERVOCOUNT = 5;
int id;
int pos;
boolean IDCheck;
boolean RunCheck;

//IO Signals-------------------
int Conveyor_Signal = 2;
int BLUE_Signal = 20;
int RED_Signal = 18;
int Robot_1_Signal_From = 12;
int Robot_1_Signal_To = 14;
int Cam_LED = 16;

//Initial Start Program-----------------------------------------------------------------
void setup(){
   pinMode(Conveyor_Signal,OUTPUT); //Main Controller Signal - Conveyor Signal
   pinMode(BLUE_Signal,OUTPUT); //Main Controller Signal - Color BLUE Signal
   pinMode(RED_Signal,OUTPUT); //Main Controller Signal - Color RED Signal
   pinMode(Robot_1_Signal_From,INPUT); //Signal from ARBOTIX Robot 1
   pinMode(Robot_1_Signal_To,OUTPUT); // Signal to ARBOTIX Robot 1
   pinMode(Cam_LED,OUTPUT); //Camera LED Light source
   
   //initial signals
   digitalWrite(Conveyor_Signal, LOW);
   digitalWrite(RED_Signal, LOW);
   digitalWrite(BLUE_Signal, LOW);
   digitalWrite(Robot_1_Signal_From, LOW);
   digitalWrite(Robot_1_Signal_To, LOW);
   digitalWrite(Cam_LED, LOW);

   //initialize variables 
   id = 1;
   pos = 0;
   IDCheck = 1;
   RunCheck = 0;
   
   pixy.init();
   
  //open serial port
   Serial.begin(9600);
   delay (500);   
   Serial.println("###########################");    
   Serial.println("Serial Communication Established.");
   Serial.println("###########################");
   
   MoveHome();
  
   MenuOptions();
   
   RunCheck = 1;
}

//Program Selection Loop----------------------------------------------------------------
void loop(){

    int inByte = Serial.read();

    switch (inByte) {

    case '1':    
      ScanServo();
      break;

    case '2':    
      RelaxServos();
      break;
      
     case '3':    
      CaseProgram();
      break;     
    }
}

//
void ScanServo(){
  id = 1;  
  Serial.println("###########################");
  Serial.println("Starting Servo Scanning Test.");
  Serial.println("###########################");
      
  while (id <= SERVOCOUNT){
  pos =  ax12GetRegister(id, 36, 2);
  Serial.print("Servo ID: ");
  Serial.println(id);
  Serial.print("Servo Position: ");
  Serial.println(pos);
  
  if (pos <= 0){
  Serial.println("###########################");
  Serial.print("ERROR! Servo ID: ");
  Serial.print(id);
  Serial.println(" not found. Please check connection and verify correct ID is set.");
  Serial.println("###########################"); 
  IDCheck = 0;
  }
  
  id = (id++)%SERVOCOUNT;
  delay(1000);
  }
  if (IDCheck == 0){
    Serial.println("###########################");
    Serial.println("ERROR! Servo ID(s) are missing from Scan. Please check connection and verify correct ID is set.");
    Serial.println("###########################");  
  }
  else{
  Serial.println("All servo IDs present.");
  }
    if (RunCheck == 1){
    MenuOptions();
  }

}

void MoveHome(){
delay(100);                    // recommended pause
    bioloid.loadPose(Home);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    Serial.println("###########################");
    Serial.println("Moving servos to Home position");
    Serial.println("###########################");    
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    if (RunCheck == 1){
      MenuOptions();
  }
}

void RelaxServos(){
  id = 1;
  Serial.println("###########################");
  Serial.println("Relaxing Servos.");
  Serial.println("###########################");    
  while(id <= SERVOCOUNT){
    Relax(id);
    id = (id++)%SERVOCOUNT;
    delay(50);
  }
   if (RunCheck == 1){
      MenuOptions();
  }
}

void CaseProgram(){
int Sequence = 0;

while (1 != 2) {
switch(Sequence) {
  
case 0: {
    while (digitalRead(Robot_1_Signal_From)==LOW) {
    delay(500);
    }
 //Position 1
   Serial.println("Case 0");
    delay(100);                    // recommended pause
    bioloid.loadPose(POS1);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 10: {
    Serial.println("Case 10");
//Position 2
    delay(100);                    // recommended pause
    bioloid.loadPose(POS2);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 20: {
    Serial.println("Case 20");
//Position 3
    delay(100);                    // recommended pause
    bioloid.loadPose(POS3);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 30: {
    Serial.println("Case 30");
//Position 4
    digitalWrite(Robot_1_Signal_To, HIGH);
    
    while (digitalRead(Robot_1_Signal_From) == LOW); { 
      delay(100);
    }
    delay(500);
    digitalWrite(Robot_1_Signal_To, LOW);
    delay(100);                    // recommended pause
    bioloid.loadPose(POS4);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}


case 40: {
    Serial.println("Case 40");
//Position 5
    delay(100);                    // recommended pause
    bioloid.loadPose(POS5);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 50: {
    Serial.println("Case 50");
//Position 6
    delay(100);                    // recommended pause
    bioloid.loadPose(POS6);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 60: {
    Serial.println("Case 60");
//Position 7
    delay(100);                    // recommended pause
    bioloid.loadPose(POS7);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}


case 70: {
   Serial.println("Case 70");
//Position 8
while(digitalRead(RED_Signal)==LOW && digitalRead(BLUE_Signal)==LOW){
  digitalWrite(Cam_LED, HIGH);
  uint16_t blocks;
  int Current_Color;
  
  // grab blocks!
  blocks = pixy.getBlocks();
  Current_Color = pixy.blocks[0].signature;
  
  // If there are detect blocks, print them!
  if (blocks && Current_Color == 1)
  {
  Serial.println(Current_Color);
  digitalWrite(RED_Signal, HIGH);
  delay(1000);
  }
  
  if (blocks && Current_Color == 2)
  {
  Serial.println(Current_Color);
  digitalWrite(BLUE_Signal, HIGH);
  delay(1000);
  }
}

digitalWrite(RED_Signal, LOW);
digitalWrite(BLUE_Signal, LOW);
digitalWrite(Cam_LED, LOW);

    delay(100);                    // recommended pause
    bioloid.loadPose(POS8);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 80: {
    Serial.println("Case 80");
//Position 9
    delay(100);                    // recommended pause
    bioloid.loadPose(POS9);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 90: {
    Serial.println("Case 90");
    digitalWrite(Conveyor_Signal, HIGH);
    delay(1000);
//Position 10
    delay(100);                    // recommended pause
    bioloid.loadPose(POS10);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 100: {
    Serial.println("Case 100");
//Position 11
    delay(100);                    // recommended pause
    bioloid.loadPose(POS11);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 110: {
    Serial.println("Case 110");
//Position 12
    delay(100);                    // recommended pause
    bioloid.loadPose(POS12);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 120: {
    Serial.println("Case 120");
//Position 13
    delay(100);                    // recommended pause
    bioloid.loadPose(POS13);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 130: {
    Serial.println("Case 130");
//Pulse for conveyor
    digitalWrite(Conveyor_Signal,LOW);
    Sequence = Sequence + 10;
    break;
}

case 140: {
   Serial.println("Case 140");
  //Position 14
    delay(100);                    // recommended pause
    bioloid.loadPose(POS14);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = Sequence + 10;
    break;
}

case 150: {
    Serial.println("Case 150");
  //Position 15
    delay(100);                    // recommended pause
    bioloid.loadPose(POS15);   // load the pose from FLASH, into the nextPose buffer
    bioloid.readPose();            // read in current servo positions to the curPose buffer
    delay(1000);
    bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
    while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
    }
    Sequence = 0;
    break;
  }
}
  Serial.println("End Switch Case");
 }
  Serial.println("End While Loop");
}



void MenuOptions(){
    Serial.println("###########################"); 
    Serial.println("Please enter option 1-5 to run individual tests again.");     
    Serial.println("1) Servo Scanning");        
    Serial.println("2) Relax Servos"); 
    Serial.println("3) Run Case Program");           
    Serial.println("###########################");
}
